/*****************************************************************************************
 *
 *  LISTENER: AN EXAMPLE FOR THE SIMPLESPEECH LIBRARY
 *
 *  IT DEMONSTRATES SPEECH SYNTHESIZER AND SPEECH RECOGNIZER
 *
 *  NOTE: CHECK THE LOCATION OF THE sample.gram FILE IN THE sample.config.xml FILE
 *  (in the 'The Grammar configuration' section)
 *
 *  AFTER THE WELCOME MESSAGE, SAY ONE OF THE FOLLOWING WORDS:
 *  RED, GREEN, BLUE, YELLOW, CIRCLE OR SQUARE
 *
 *  Tested  with Processing 3.4 (32- and 64 bit)
 *
 *  Release: 10/23/2018
 *
 *  (c) 2018 Rolf van Gelder / CAGE Web Design
 * 
 *****************************************************************************************/
import simpleSpeech.*;

// LISTENER
Listen listener;

// SPEAKER
Speak  talker;

boolean debug = false;
boolean intro = false;

PImage  red_circle;
PImage  green_circle;

String  lastRecognized = "(waiting)";

color   currentColor = color(255, 0, 0);
String  currentShape = "circle";


/*****************************************************************************************
 *
 *  SETUP
 * 
 *****************************************************************************************/
void setup() {
  size(600, 600);
  background(0);

  // LAYOUT
  textAlign(CENTER);
  rectMode(CENTER);
  textSize(36);

  listener = new Listen(this, "\\sample.config.xml");
  talker   = new Speak(this, "kevin16");

  talker.version();
  talker.listVoices();
} // setup()


/*****************************************************************************************
 *
 *  DRAW
 * 
 *****************************************************************************************/
void draw() {
  background(0);

  if (!intro) {
    // INTRO MESSAGE (ONCE)
    talker.speakAndWait("This is an example for the SimpleSpeech library");
    intro = true;
  }

  // DISPLAY THE SHAPE
  fill(currentColor);
  if (currentShape.equals("circle")) {
    ellipse(width>>1, height>>1, width>>1, width>>1);
  } else {
    rect(width>>1, height>>1, width>>1, width>>1);
  }

  pushStyle();
  // DISPLAY LAST RECOGNIZED TEXT
  fill(255);
  text(lastRecognized, width>>1, 40);
  // IS THE MICROPHONE RECORDING?
  if (listener.isRecording()) {
    fill(0, 255, 0);
  } else
  {
    fill(255, 0, 0);
  } // if (listener.isRecording())
  ellipse(30, 30, 15, 15);  
  popStyle();
} // draw()


/*****************************************************************************************
 *
 *  LISTENER EVENT
 * 
 *****************************************************************************************/
void listenEvent(Listen _l) {

  // DON'T LISTEN WHILE SPEAKING!
  if (talker.isSpeaking()) return;

  String s = _l.readString();

  if (s != null) {
    if (debug) {
      println("heard: " + s);
    } // if (debug)

    if (s.equals("red")) {
      currentColor = color(255, 0, 0);
    } else if (s.equals("green")) {
      currentColor = color(0, 255, 0);
    } else if (s.equals("blue")) {
      currentColor = color(0, 0, 255);
    } else if (s.equals("yellow")) {
      currentColor = color(255, 255, 0);
    } else if (s.equals("circle")) {
      currentShape = "circle";
    } else if (s.equals("square")) {
      currentShape = "square";
    } // if (s.equals("red"))

    lastRecognized = s;
  } // if (s != null)
} // listenEvent()


/*****************************************************************************************
 *
 *  STOP SKETCH
 * 
 *****************************************************************************************/
void stop()
{
  // SHUT DOWN GRACEFULLY...
  talker.dispose();
  listener.dispose();

  super.stop();
}
