/*************************************************************************************
 *
 *	Simple Speech: a Speech Synthesizer and Voice Recognizer for Processing
 *
 *	v2.0 - 10/23/2018
 *
 *	(c) 2018 -  Rolf van Gelder, CAGE Web Design, Eindhoven, NL (cagewebdev.com)
 *
 *************************************************************************************/
package simpleSpeech;

import processing.core.*;
import edu.cmu.sphinx.frontend.util.Microphone;
import edu.cmu.sphinx.recognizer.Recognizer;
import edu.cmu.sphinx.result.Result;
import edu.cmu.sphinx.util.props.ConfigurationManager;
import edu.cmu.sphinx.util.props.PropertyException;

import java.io.IOException;
import java.net.URL;
import java.lang.reflect.*;

/*************************************************************************************
 *
 *	Listen: a Class for speech recognition (using the Sphinx engine
 *
 *************************************************************************************/ 
public class Listen implements Runnable {

	Recognizer recognizer;
	Microphone microphone;
	PApplet parent;
	Method listenEventMethod;
	Thread t;
	String resultText;
	String config;
	
	// VERSION OF THE LIBRARY
	String version     = "2.0";
	String releaseDate = "10/23/2018";		

	/**********************************************************************************
	 *
	 *	Constructor: instantiate the class in procesing's standard way: pass "this" as
	 *	a parameter, when instantiating in a Processing applet
	 *
	 *	@param _p	Processing applet
	 *	@param _c	name of the configuration file (e.g. sample.config.xml)
	 *
	 **********************************************************************************/	
	public Listen(PApplet _p, String _c) {
		this.parent = _p;
		this.config = _c;
		parent.registerMethod("dispose", this);
		resultText = "";
		init();
		t = new Thread(this);
		t.start();
	} // Listen()

	
	/**********************************************************************************
	 *
	 *	Initialize the Recognizer
	 *
	 **********************************************************************************/
	private void init() {
		try {
			listenEventMethod = parent.getClass().getMethod("listenEvent",
					new Class[] { Listen.class });
		} catch (Exception e) {
		}

		String path = parent.dataPath("");

		try {
			URL url = new URL("file:///" + path + config);
			ConfigurationManager cm = new ConfigurationManager(url);
			recognizer = (Recognizer) cm.lookup("recognizer");
			microphone = (Microphone) cm.lookup("microphone");
			recognizer.allocate();
		} catch (Exception e) {
		}
		microphone.startRecording();
	} // init()

	
	/**********************************************************************************
	 *
	 *	Run the Recognizer
	 *
	 **********************************************************************************/
	public void run() {
		Result result;
		while (true) {
			try {
				result = recognizer.recognize();
				if (result != null) {
					resultText = result.getBestFinalResultNoFiller();
					if (resultText.length() > 0) {
						makeEvent();
					} // if (resultText.length()
				} // if (result != null)				
			} catch (Exception e) {
			}

		} // while (true)
	} // run()


	/**********************************************************************************
	 *
	 *	Returns a string with the recognised text
	 *
	 *	It should be called within the listenEvent() handler<br />
	 *	<font color="navy">
	 *		<code>
	 *			void listenEvent(Listen _l) {<br />
	 *			&nbsp;&nbsp;&nbsp;// GET RECOGNISED TEXT<br />
	 *			&nbsp;&nbsp;&nbsp;stringRead = _l.readString();<br />
	 *			}<br />
	 *		</code>
	 *	</font>
	 *
	 *	@return recognized text
	 *
	 **********************************************************************************/
	public String readString() {
		return resultText;
	} // readString()

	
	/**********************************************************************************
	 *
	 *	Create the Listen Event
	 *
	 **********************************************************************************/
	public void makeEvent() {
		try {
			if (listenEventMethod != null) {
				try {
					listenEventMethod.invoke(parent, new Object[] { this });
				} catch (Exception e) {
					listenEventMethod = null;
				} // try
			} // if (listenEventMethod != null)			
		} catch (Exception e) {
		}
	} // makeEvent()
	

	/*********************************************************************************
	 *
	 *	Is the Microphone recording?
	 *
	 *	@return Is the Microphone recording?
	 *
	 *********************************************************************************/
	public boolean isRecording(){
		return microphone.isRecording();
	} // isRecording()

	
	/*********************************************************************************
	 *
	 *	Displays Version Information Of This Library In The Console
	 *
	 *********************************************************************************/
	public void version() {
		System.out.println("\nSimpleSpeech v" + version + " [" + releaseDate + "], by Rolf van Gelder (cagewebdev.com)\n");
	} // void version()	

	
	/**********************************************************************************
	 *
	 *	Dispose the Recognizer
	 *
	 **********************************************************************************/
	public void dispose() {
		try {
			recognizer.deallocate();
		} catch (Exception e) {
		}
	} // dispose()
} // Listen